# Copyright (C) 2011, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pwd
import urlparse
from os.path import join, exists, dirname
from gettext import gettext as _

import gconf
import dbus.service

from sugar_client import util


VERSION = '1.0'

XO_SERIAL_PATH = '/ofw/mfg-data/SN'
XO_UUID_PATH = '/ofw/mfg-data/U#'

debug = util.Option(
        _('debug logging level; multiple argument'),
        default=0, type_cast=int, short_option='-D', action='count')

uid = util.Option(
        _('current user\'s unique identity; needs to be set only from ' \
                'profile configuration file; will be auto-created ' \
                'on initial registration'))

schoolserver = util.Option(
        _('school server\'s FQDN, will be auto-set after registration'))


def _api_url_cast(url):
    if not url.startswith('http://') and not url.startswith('https://'):
        url = 'http://' + url
    url = urlparse.urlparse(url)
    # pylint: disable-msg=E1101
    return '%s://%s' % (url.scheme, url.netloc)


api_url = util.Option(
        _('url to connect to school server\'s RESTfull API'),
        default='http://schoolserver:8000', type_cast=_api_url_cast,
        short_option='-a')

auto_register = util.Option(
        _('process auto-registeration on a school server on startup'),
        default=0, type_cast=int, action='count')

jabber_url = util.Option(
        _('jabber server for Sugar collaboraiton'))

backup_url = util.Option(
        _('rsync url to backup/restore Journal'))

backup_timeout = util.Option(
        _('if more than 0, do regular backups with specified delay ' \
                'in seconds'),
        default=0, type_cast=int)

auto_restore = util.Option(
        _('process auto-restore if there is pending one on the server'),
        default=False, action='store_true')

update_args = util.Option(
        _('optional command-line arguments to pass to packager ' \
                'while updating the system'))

update_timeout = util.Option(
        _('if more than 0, do regular unattended updates with specified ' \
                'delay in seconds'),
        default=0, type_cast=int)


def profile_path(*args):
    """Path within sugar profile directory.

    Missed directories will be created.

    :param args:
        path parts that will be added to the resulting path
    :returns:
        full path with directory part existed

    """
    if os.geteuid():
        root_dir = join(os.environ['HOME'], '.sugar',
                os.environ.get('SUGAR_PROFILE', 'default'))
    else:
        root_dir = '/var/sugar-client'
    result = join(root_dir, *args)
    if not exists(dirname(result)):
        os.makedirs(dirname(result))
    return result


def machine_sn():
    """Machine's Serial Number value.

    :returns:
        machine serial number if it can be obtained

    """
    if exists(XO_SERIAL_PATH):
        return _read_XO_value(XO_SERIAL_PATH)


def machine_uuid():
    """Machine's UUID value.

    :returns:
        machine UUID if it can be obtained

    """
    if exists(XO_UUID_PATH):
        return _read_XO_value(XO_UUID_PATH)


def nick():
    gconf_client = gconf.client_get_default()
    return gconf_client.get_string('/desktop/sugar/user/nick')


class LocalService(object):

    def __init__(self, bus_class, name, path):
        pass

    def remove_from_connection(self, connection=None, path=None):
        pass

    @staticmethod
    def signal(*args, **kwargs):

        def decorate(f):
            return f

        return decorate

    @staticmethod
    def method(*args, **kwargs):

        def decorate(f):
            return f

        return decorate

    def get_unix_user(self, sender_name):
        return pwd.getpwuid(os.getuid()).pw_name


class DBusService(dbus.service.Object):

    def __init__(self, bus_class, name, path):
        bus = bus_class()
        if name:
            bus_name = dbus.service.BusName(name, bus=bus)
            dbus.service.Object.__init__(self, bus_name, path)
        else:
            dbus.service.Object.__init__(self, bus, path)

    @staticmethod
    def signal(*args, **kwargs):
        return dbus.service.signal(*args, **kwargs)

    @staticmethod
    def method(*args, **kwargs):
        return dbus.service.method(*args, **kwargs)

    def get_unix_user(self, sender_name):
        return pwd.getpwuid(self.connection.get_unix_user(sender_name)).pw_name


def _read_XO_value(path):
    return file(path).read().rstrip('\0\n')


BaseService = LocalService

util.Option.seek('main')
util.Command.seek('main')
